@@ -18,6 +18,7 @@ from .media_helper import MediaHelper
 from .singleton import Singleton
 from .subtitle import Subtitle
 from .hyperparameters import Hyperparameters
+from .translator import Translator
 from .exception import TerminalException
 from .exception import NoFrameRateException
 from .logger import Logger
@@ -38,7 +39,7 @@ class Predictor(Singleton):
 
     __SEGMENT_PREDICTION_TIMEOUT = 60  # Maximum waiting time in seconds when predicting each segment
 
-    def __init__(self, **kwargs):
+    def __init__(self, **kwargs) -> None:
         """Feature predictor initialiser.
 
             Keyword Arguments:
@@ -82,7 +83,7 @@ class Predictor(Singleton):
                 self.__feature_embedder.step_sample = 1 / frame_rate
                 self.__on_frame_timecodes(subs)
             except NoFrameRateException:
-                Predictor.__LOGGER.warning("Cannot find frame rate for %s" % video_file_path)
+                Predictor.__LOGGER.warning("Cannot detect the frame rate for %s" % video_file_path)
             return subs, audio_file_path, voice_probabilities, frame_rate
         finally:
             if os.path.exists(audio_file_path):
@@ -132,7 +133,7 @@ class Predictor(Singleton):
                 self.__feature_embedder.step_sample = 1 / frame_rate
                 self.__on_frame_timecodes(new_subs)
             except NoFrameRateException:
-                Predictor.__LOGGER.warning("Cannot find frame rate for %s" % video_file_path)
+                Predictor.__LOGGER.warning("Cannot detect the frame rate for %s" % video_file_path)
             Predictor.__LOGGER.debug("Aligned segments generated")
             return new_subs, subs, voice_probabilities, frame_rate
         finally:
@@ -304,6 +305,7 @@ class Predictor(Singleton):
                     self.__cancel_futures(futures[i:], Predictor.__SEGMENT_PREDICTION_TIMEOUT)
                     message = "Exception on segment alignment: {}\n{}".format(str(e), "".join(traceback.format_stack()))
                     Predictor.__LOGGER.error(message)
+                    traceback.print_tb(e.__traceback__)
                     if isinstance(e, TerminalException):
                         raise e
                     else:
@@ -384,6 +386,7 @@ class Predictor(Singleton):
                     thread_name, segment_index, str(e), "".join(traceback.format_stack())
                 )
             )
+            traceback.print_tb(e.__traceback__)
             if exit_segfail:
                 raise TerminalException("At least one of the segments failed on alignment. Exiting...") from e
             return subs[segment_index]
@@ -490,6 +493,7 @@ class Predictor(Singleton):
                 tee = True
             if Logger.QUIET:
                 tee = False
+
             with self.__lock:
                 # Execute the task
                 ExecuteTask(
@@ -514,6 +518,8 @@ class Predictor(Singleton):
                 )
             )
             return adjusted_subs
+        except KeyboardInterrupt:
+            raise TerminalException("Subtitle stretch interrupted by the user")
         finally:
             # Housekeep intermediate files
             if task.audio_file_path_absolute is not None and os.path.exists(
@@ -569,7 +575,7 @@ class Predictor(Singleton):
             audio_file_path {string} -- The file path of the original audio (default: {None}).
             subtitles {list} -- The list of SubRip files (default: {None}).
             max_shift_secs {float} -- The maximum seconds by which subtitle cues can be shifted (default: {None}).
-            previous_gap {float} -- The duration betwee the start time of the audio segment and the start time of the subtitle segment.
+            previous_gap {float} -- The duration between the start time of the audio segment and the start time of the subtitle segment (default: {None}).
 
         Returns:
             tuple -- The shifted subtitles, the audio file path and the voice probabilities of the original audio.
@@ -630,6 +636,7 @@ class Predictor(Singleton):
                 voice_probabilities = self.__network.get_predictions(train_data, weights_file_path)
             except Exception as e:
                 Predictor.__LOGGER.error("[{}] Prediction failed: {}\n{}".format(thread_name, str(e), "".join(traceback.format_stack())))
+                traceback.print_tb(e.__traceback__)
                 raise TerminalException("Prediction failed") from e
             finally:
                 del train_data
@@ -669,7 +676,7 @@ class Predictor(Singleton):
                 self.__feature_embedder.position_to_duration(pos_to_delay) - original_start
             )
         elif subtitles is not None:  # for each in second pass
-            seconds_to_shift = self.__feature_embedder.position_to_duration(pos_to_delay) - previous_gap
+            seconds_to_shift = self.__feature_embedder.position_to_duration(pos_to_delay) - previous_gap if previous_gap is not None else 0.0
         else:
             if os.path.exists(audio_file_path):
                 os.remove(audio_file_path)
